<?php

namespace FedaPay;

/**
 * Class ApiKey
 *
 * @property int $id
 * @property string $public_key
 * @property string $private_key
 * @property string $created_at
 * @property string $updated_at
 *
 * @package FedaPay
 */
class ApiKey extends Resource
{

}
